unit dnDBPxG;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  Component TDBPxGrid:
    This is an enhanced DBGrid, which adds support for Paradox-style masks (and
    fixes some "features", where InplaceEditor resets EditText). This took some
    trial and error, because Borland didn't implemented DBGrid's InplaceEditor
    in the interface part. I didn't want to copy Borland's DBGrids.PAS, so I had
    to crack DBGrid and replace it's DataLink with a special version.
    So if you want to use this sourcecode with your favorite DBGrid, please
    check if they replaced DataLink too and try to change sourcecode/inheritance
    accordingly (e.g. it works with RXDBGrid, if you change class-definition:
    TDBPxGrid = class(TRxDBGrid)).

    new properties:
      PxMask (protected)

    It still uses Borland's EditMask (so PLEASE DO NOT SUPPLY BOTH EditMask and
    PxMask) and validates them.

******************************************************************************)

interface

{$I dnInc.INC}

uses
  Windows, Grids, Mask, Controls, Graphics, Classes, Messages, DB, DBTables,
  DBGrids, Forms, Consts,
  dnPxMask, dnPxGrid, dnDBPxM;

type
  {*** The default TGridDataLink is replaced with this}
  TPxGridDataLink = class(TGridDataLink)
  private
  protected
    procedure DataSetChanged; override;
    procedure UpdateData; override;
  public
  end;

  TDBPxGrid = class(TDBGrid)
  private
    FPxMask: TGridPxMask;
    procedure SetPxMask(value: TGridPxMask);
    function GetOnValidChanged: TValidChangedEvent;
    procedure SetOnValidChanged(value: TValidChangedEvent);
  protected
    procedure KeyPress(var Key: Char); override;
    procedure SetEditText(ACol, ARow: Longint; const value: string); override;
    function GetEditText(ACol, ARow: Longint): string; override;
    function GetEditMask(ACol, ARow: Longint): string; override;
    function SelectCell(ACol, ARow: Longint): Boolean; override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property PxMask: TGridPxMask read FPxMask write SetPxMask stored false;
      //must be published

    property OnValidChanged: TValidChangedEvent read GetOnValidChanged
      write SetOnValidChanged;
  end;

implementation

uses
  SysUtils, TypInfo,
  dnUtil;

{*** This class-definition is only needed to access a inherited private Var:
  FGrid}
type
  TDummyGridDataLink = class(TDataLink)
  private
    FGrid: TCustomDBGrid;
  end;

procedure TPxGridDataLink.DataSetChanged;
begin
  inherited;
  TDBPxGrid(TDummyGridDataLink(self).FGrid).PxMask.ErrorPos := 0;
end;

procedure TPxGridDataLink.UpdateData;
begin
  with TDummyGridDataLink(self), TDBPxGrid(FGrid).PxMask do
  try
    include(FState, pxReEnter);
    if EditModified and AutoValidate then
      ValidateEdit;

    inherited UpdateData;

    exclude(FState, pxReEnter);
  except
    PrepareShowError;
    raise;
  end;
end;

{*** TDBPxGrid ***}
constructor TDBPxGrid.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FPxMask := TGridPxMask.Create(self, false); //MUST be created after inherited
  FakeInheritance(DataLink, TPxGridDataLink);
end;

destructor TDBPxGrid.Destroy;
begin
  if assigned(FPxMask) then FPxMask.free;
  inherited Destroy;
end;

procedure TDBPxGrid.KeyPress(var Key: Char);
begin
  FPxMask.CheckResetKey(Key);
  FPxMask.HookKeyPress(Key);
  inherited KeyPress(Key);
end;

function TDBPxGrid.SelectCell(ACol, ARow: Longint): Boolean;
begin
  Result := inherited SelectCell(ACol, ARow);
end;

procedure TDBPxGrid.SetEditText(ACol, ARow: Longint; const value: string);
begin
  if not (pxReEnter in FPxMask.FState) then
    inherited;
end;

function TDBPxGrid.GetEditText(ACol, ARow: Longint): string;
begin
  if (pxReEnter in FPxMask.FState) then
  begin
    Result := InplaceEditor.Text;
    exclude(FPxMask.FState, pxReEnter);
  end
  else
    Result := inherited GetEditText(ACol, ARow);
end;

function TDBPxGrid.GetEditMask(ACol, ARow: Longint): string;
var
  pxm: TPxMask;
begin
  Result := inherited GetEditMask(ACol, ARow);
  if HasPxMask(SelectedField, pxm) then
    FPxMask.assign(pxm);
end;

procedure TDBPxGrid.SetPxMask(value: TGridPxMask);
begin
  FPxMask.assign(value);
end;

function TDBPxGrid.GetOnValidChanged: TValidChangedEvent;
begin
  Result := FPxMask.OnValidChanged;
end;

procedure TDBPxGrid.SetOnValidChanged(value: TValidChangedEvent);
begin
  FPxMask.OnValidChanged := value;
end;

end.
