unit dnUtil;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

some internal Utility routines
******************************************************************************)

{$I dnInc.INC}

interface
uses
  TypInfo, classes;

const
  StringProps = [tkString, tkLString, tkWString];
  OrdProps = [tkInteger, tkChar, tkEnumeration, tkSet, tkClass];
  ClassProp = [tkClass];
  BoolProp = [tkEnumeration];
  FloatProps = [tkFloat];
  AnyProps = [low(TTypeKind)..High(TTypeKind)] - [low(TTypeKind)];
{ AnyProps = [succ(low(TTypeKind))..High(TTypeKind)]; <-- this caused some strange compiler error}

function GetProperty(Instance: TPersistent;
  const PropName: string;
  PropKinds: TTypeKinds;
  var Value: variant): boolean;

function SetProperty(Instance: TPersistent;
  const PropName: string;
  PropKinds: TTypeKinds;
  Value: variant): boolean;

function HasProperty(Instance: TObject;
  const PropName: string;
  PropKind: TTypeKinds;
  var PropInfo: PPropInfo): boolean;

function GetBoolProperty(Instance: TPersistent;
  const PropName: string;
  var Value: boolean): boolean;

function GetOrdProperty(Instance: TPersistent;
  const PropName: string;
  PropKind: TTypeKind;
  var Value: Longint): boolean;

function GetStringProperty(Instance: TPersistent;
  const PropName: string;
  var Value: string): boolean;

procedure FakeInheritance(Instance: TObject; FakeClassParent: TClass);

implementation

uses
  SysUtils;

function HasProperty(Instance: TObject;
  const PropName: string;
  PropKind: TTypeKinds;
  var PropInfo: PPropInfo): boolean;
begin
  result := false;
  if assigned(Instance) then
  begin
    PropInfo := GetPropInfo(Instance.ClassInfo, PropName);
    result := (PropInfo <> nil) and (PropInfo^.PropType^.Kind in PropKind);
  end
  else
    PropInfo := nil;
end;

function GetBoolProperty(Instance: TPersistent;
  const PropName: string;
  var Value: boolean): boolean;
var
  PropInfo: PPropInfo;
begin
  result := false;
  Value := false;
  if not HasProperty(Instance, PropName, BoolProp, PropInfo) then exit;
  Value := boolean(GetOrdProp(Instance, PropInfo));
  result := true;
end;

function GetStringProperty(Instance: TPersistent;
  const PropName: string;
  var Value: string): boolean;
var
  PropInfo: PPropInfo;
begin
  result := false;
  Value := '';
  if not HasProperty(Instance, PropName, StringProps, PropInfo) then exit;
  Value := (GetStrProp(Instance, PropInfo));
  result := true;
end;

function GetOrdProperty(Instance: TPersistent;
  const PropName: string;
  PropKind: TTypeKind;
  var Value: Longint): boolean;
var
  PropInfo: PPropInfo;
begin
  result := false;
  Value := 0;
  if not HasProperty(Instance, PropName, [PropKind], PropInfo) then exit;
  Value := GetOrdProp(Instance, PropInfo);
  result := true;
end;

function GetProperty(Instance: TPersistent;
  const PropName: string;
  PropKinds: TTypeKinds;
  var Value: variant): boolean;
var
  PropInfo: PPropInfo;
begin
  VarClear(Value);
  if assigned(Instance) then
  try
    PropInfo := GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in PropKinds) then
      case PropInfo^.PropType^.Kind of
        tkInteger, tkChar, tkEnumeration, tkSet, tkClass:
          Value := GetOrdProp(Instance, PropInfo);
        tkFloat:
          Value := GetFloatProp(Instance, PropInfo);
        tkString, tkLString, tkWString:
          Value := GetStrProp(Instance, PropInfo);
        tkVariant:
          Value := GetVariantProp(Instance, PropInfo);
      end;
  except
  end;
  result := not VarIsEmpty(Value);
end;

function SetProperty(Instance: TPersistent;
  const PropName: string;
  PropKinds: TTypeKinds;
  Value: variant): boolean;
var
  PropInfo: PPropInfo;
begin
  result := false;
  if assigned(Instance) then
  begin
    PropInfo := GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in PropKinds) then
      case PropInfo^.PropType^.Kind of
        tkInteger, tkChar, tkEnumeration, tkSet, tkClass:
          SetOrdProp(Instance, PropInfo, Value);
        tkFloat:
          SetFloatProp(Instance, PropInfo, Value);
        tkString, tkLString, tkWString:
          SetStrProp(Instance, PropInfo, Value);
        tkVariant:
          SetVariantProp(Instance, PropInfo, Value);
      else
        exit;
      end;
    result := true;
  end;
end;

const
  SNoClassParent = '%s is not the ClassParent of %s';

procedure FakeInheritance(Instance: TObject; FakeClassParent: TClass);
begin
  if Instance.ClassType <> FakeClassParent.ClassParent then
    Exception.createFmt(SNoClassParent,
      [FakeClassParent.ClassName, Instance.ClassName]);
  Pointer(Pointer(Instance)^) := FakeClassParent;
end;

end.
